%%%%%%%%%%%%%%%%%%%%%%
%  Test code for the paper:  
%  X. Fu, J. Huang, X. Ding, Y. Liao and J. Paisley. "Clearing the Skies: A deep network architecture for single-image rain removal", IEEE Transactions on Image Processing, 2017. 
%
%  NOTE: The MatConvNet toolbox has been installed with CUDA 7.5 and 64-bit windows 10
%        You may need to re-install MatConvNet for your own computer configuration: http://www.vlfeat.org/matconvnet/
%        This code is for demo only and the result can be slightly different from the paper due to transferring across platforms
%%%%%%%%%%%%%%%%%%%%%%

clc;
clear;

addpath '.\fast-guided-filter-code-v1'
run '.\matconvnet\matlab\vl_setupnn'

load('DerainNet.mat'); % trained model

use_gpu = 1; % GPU: 1, CPU: 0

max_patch_size = 120; % If you have a Nvidia GPU and encounter an "out of memory", make "max_patch_size" smaller.

%% parameters of guidedfilter
r = 16;
eps = 2; 
s = 4;

%% synthetic image
input = im2double(imread('.\image\synthetic\4rain.jpg'));

base_layer = zeros(size(input)); % base layer
base_layer(:, :, 1) = fastguidedfilter(input(:, :, 1), input(:, :, 1), r, eps, s);
base_layer(:, :, 2) = fastguidedfilter(input(:, :, 2), input(:, :, 2), r, eps, s);
base_layer(:, :, 3) = fastguidedfilter(input(:, :, 3), input(:, :, 3), r, eps, s);

detail_layer = input - base_layer; % detail layer

derained_detail_layer  = processing( detail_layer, model, max_patch_size, use_gpu ); % perform de-raining

final_output = base_layer + derained_detail_layer;  % final output

figure,imshow([input, final_output]); title('Left: rainy image       Right: de-rained result');


%% real-world image
input = im2double(imread('.\image\real_world\1.jpg'));

base_layer = zeros(size(input)); % base layer
base_layer(:, :, 1) = fastguidedfilter(input(:, :, 1), input(:, :, 1), r, eps, s);
base_layer(:, :, 2) = fastguidedfilter(input(:, :, 2), input(:, :, 2), r, eps, s);
base_layer(:, :, 3) = fastguidedfilter(input(:, :, 3), input(:, :, 3), r, eps, s);

detail_layer = input - base_layer;

derained_detail_layer  = processing( detail_layer, model, max_patch_size, use_gpu );

%%% enhancement
base_layer = 1./(1+exp(0.03*(-base_layer*255+170))); % enhance base layer
final_output = base_layer + 2 * derained_detail_layer; % enhance detail layer
%%%

figure,imshow([input, final_output]); title('Left: rainy image       Right: de-rained result');